from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.translation import gettext_lazy as _

from aleksis.core.mixins import ExtensibleModel


class Client(ExtensibleModel):
    name = models.CharField(verbose_name=_("Name"))

    class Meta:
        constraints = [
            models.UniqueConstraint(fields=["name", "site"], name="uniq_client_per_site")
        ]


class Subject(ExtensibleModel):
    content_type = models.ForeignKey(ContetType, on_delete=models.SET_NULL)
    object_id = models.PositiveIntegerField()
    related = GenericForeignKey("content_type", "object_id")

    def get_address(self):
        if hasattr(self.related, "get_address"):
            return self.related.get_address()
        elif hasattr(self.related, "addressing_name") and hasattr(self.related, "street") and hasattr(self.related, "housenumber") and hasattr(self.related, "postal_code") and hasattr(self.related, "place"):
            # FIXME Use proper locale-based address generation
            return f"{self.related.addressing_name}\n{self.related.street} {self.related.housenumber}\n{self.related.postal_code} {self.related.place}"

    @classmethod
    def from_person(cls, person):
        ct = ContentType.objects.get_for_model(person)
        subject, created = cls.objects.get_or_create(content_type=ct, object_id=person)

        return subject
